include ..\disk.nfo
include ..\royale.nfo
FAT_SEC=(HEADS*CYLINDRES*SECTORS)/256+1

;   1 Sector   -> BOOTSECTOR+INFODISK
;  ?? Sectorz  -> FAT 16 BITS <= 256 sectors
;  ?? Sectorz  -> 1rst FILELIST

main segment byte
     assume cs:main;ds:main

org 100h


debut:

;*** BOOTSEC ***
  mov ax,301h
  mov bx,offset bootsec
  mov cx,1
  mov dx,DISK
  call int13h
  mov ax,301h
  call int13h
  jc erreur

;*** EMPTY FAT  ***
  mov cx,256
  mov ax,FAT_EMPTY
  mov di,offset buffer
  rep stosw
  mov ax,1
FAT_INIT:
  push ax
  call DSK_nb2sector
  mov bx,offset buffer
  mov ah,3h
  call int13h
  pop ax
  jc erreur
  inc ax
  cmp ax,FAT_SEC
  jna FAT_INIT          ; empty fat (secteurs 1..FAT_SEC)

;*** EMPTY FILELIST ***
  mov byte ptr ds:[buffer],0
  mov ax,FAT_SEC+1
  call DSK_nb2sector
  mov bx,offset buffer
  mov ah,3h
  call int13h

;*** MARK EOF FILELIST ***
  mov ax,FAT_EOF
  mov di,offset buffer
  stosw
  mov ax,1
  call DSK_nb2sector
  mov bx,offset buffer
  mov ah,3h
  call int13h


;*** done ***
  mov si,offset S_OK
  jmp done
erreur:
  mov si,offset S_Erreur
done:
  call texte
  mov ax,4C00h
  int 21h

S_OK db 13,10,'Disque OK !!!',13,10,0
S_Erreur db 13,10,'Erreur !!!',13,10,7,0


int13h:
  int 13h
  mov al,'o'
  jnb int13hend
  mov al,'x'
int13hend:
  mov ah,0eh
  int 10h
ret

;******** BOOTSECTOR STARTS HERE *********

org 7c00h

Bootsec:
  jmp boot_start
  signature   db 'Royale OS.'
  nbheads     db HEADS              ; DH face/tete
  nbcylindres dw CYLINDRES          ; CH piste/cylindre
  nbsectors   db SECTORS            ; CL secteur

boot_start:
  cli
  xor ax,ax
  mov ds,ax
  mov es,ax
  mov ss,ax
  mov sp,offset bootsec
  sti

  mov si,offset S_Loading
  call texte

  mov ax,FAT_SEC+1
  call DSK_nb2sector
  mov bx,offset buffer
  mov ah,2
  int 13h

  mov si,offset buffer
  mov di,offset F_Kernel
  mov cx,28
  repz cmpsb
  jnz no_kernel
  add si,2
  lodsw                         ; ax sector !
  les bx,dword ptr cs:[Adresse]

  kernel_load:
    cmp ax,FAT_EOF
    je kernel_loaded

    push ax                       ; load file...
    add ax,FAT_SEC+1
    call DSK_nb2sector
    mov ah,2h
    int 13h
    add bx,512
    pop ax

    call DSK_chainage
    jmp kernel_load

kernel_loaded:
  mov si,offset S_done
  call texte
  mov dx,DISK
  push dx                       ; push drive for kernel
  jmp dword ptr cs:[Adresse]

no_kernel:
  mov si,offset S_NoKernel
  call texte
  jmp $

Adresse dw Ofs_OS,Seg_OS
S_Loading  db 'Chargement Kernel : ',0
S_NoKernel db 'non trouv !',7,0
S_done     db 'termin.',13,10,13,10,0
F_Kernel   db 'Kernel.com',18 dup (0)

texte:
  lodsb
  or al,al
  jz texte_end
  mov ah,0eh
  int 10h
  jmp texte
texte_end:
  ret

; dl : n de disque (0,1,80h,81h)
; dh : tte (0..N-1)
; ch : cylindre/piste (0..N-1)
; cl : secteur (1..N)
; al : nombre de secteur
DSK_nb2sector:
  push bx
  xor dx,dx
  xor ch,ch
  mov cl,SECTORS
  div cx
  inc dl
  mov cl,dl ;save sector here
  xor dx,dx
  mov bx,CYLINDRES
  div bx
  mov ch,dl ;cylindre
  shl dh,6  ;DH poids fort  (XX000000b)
  or cl,dh
  mov dh,al ;head
  mov dl,DISK
  mov al,1
  pop bx
ret

; Entree : ax=FAT_nb
; Sortie : ax=new FAT_nb

Old_Fat_sector dw 0

DSK_chainage:
  xor dx,dx                     ; next sector !
  mov cx,256
  div cx           ; sector/256
  shl dx,1
  mov si,offset buffer
  add si,dx        ; si : address of new sector !
  inc ax           ; sector/256+1 (bootsector) -> fat cluster we want !
  cmp ax,ds:[old_fat_sector]
  je same_fat_sector
  mov ds:[old_fat_sector],ax
  push es bx
  push ds
  pop es
  call DSK_nb2sector
  mov bx,offset buffer
  mov ah,2h
  int 13h
  pop bx es
same_fat_sector:
  lodsw                         ; ax sector !
ret

;*** BUFFER DISQUE ***
Buffer:

org 7dfeh
dw 0aa55h

main ends
end debut
